package dev.fitko.fitconnect.client.subscriber;

import dev.fitko.fitconnect.api.domain.model.event.SubmissionStatus;
import dev.fitko.fitconnect.api.domain.model.metadata.AdditionalReferenceInfo;
import dev.fitko.fitconnect.api.domain.model.metadata.Metadata;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.domain.subscriber.ReceivedSubmission;
import dev.fitko.fitconnect.api.domain.subscriber.ReceivedSubmissionData;
import org.junit.jupiter.api.Test;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertTrue;

class ReceivedSubmissionTest {

    @Test
    void testApplicationDateCouldNotBeParsed() {

        // Given
        final var additionalReferenceInfo = new AdditionalReferenceInfo();
        additionalReferenceInfo.setApplicationDate("2020-14-01 99:00:00");

        var metadata = new Metadata();
        metadata.setAdditionalReferenceInfo(additionalReferenceInfo);

        var receivedData = new ReceivedSubmissionData("data", metadata, new Submission(), new SubmissionStatus(), List.of(), Map.of());
        var receivedSubmission = new ReceivedSubmission(null, receivedData);

        // When
        final Optional<LocalDate> applicationDate = receivedSubmission.getApplicationDate();

        // Then
        assertTrue(applicationDate.isEmpty());
    }

    @Test
    void testNoApplicationDatePresent() {

        // Given
        var metadata = new Metadata();
        metadata.setAdditionalReferenceInfo( new AdditionalReferenceInfo());

        var receivedData = new ReceivedSubmissionData("data", metadata, new Submission(), new SubmissionStatus(), List.of(), Map.of());
        var receivedSubmission = new ReceivedSubmission(null, receivedData);

        // When
        final Optional<LocalDate> applicationDate = receivedSubmission.getApplicationDate();

        // Then
        assertTrue(applicationDate.isEmpty());
    }

    @Test
    void testNoAdditionalReferenceInfoPresent() {

        // Given
        var receivedData = new ReceivedSubmissionData("data", new Metadata(), new Submission(), new SubmissionStatus(), List.of(), Map.of());
        var receivedSubmission = new ReceivedSubmission(null, receivedData);

        // When
        final Optional<LocalDate> applicationDate = receivedSubmission.getApplicationDate();

        // Then
        assertTrue(applicationDate.isEmpty());
    }
}